diff --git a/config/chroot_local-includes/usr/local/lib/tails-shell-library/common.sh b/config/chroot_local-includes/usr/local/lib/tails-shell-library/common.sh
index b7b6338..f1dce53 100644
--- a/config/chroot_local-includes/usr/local/lib/tails-shell-library/common.sh
+++ b/config/chroot_local-includes/usr/local/lib/tails-shell-library/common.sh
@@ -1,5 +1,14 @@
 #!/bin/sh
 
+# Get monotonic time in seconds. See clock_gettime(2) for details.
+# Note: we limit ourselves to seconds simply because floating point
+# arithmetic is a PITA in the shell.
+clock_gettime_monotonic() {
+    perl -w -MTime::HiRes=clock_gettime,CLOCK_MONOTONIC \
+         -E 'say clock_gettime(CLOCK_MONOTONIC)' | \
+        sed 's/\..*$//'
+}
+
 # Run `check_expr` until `timeout` seconds has passed, and sleep
 # `delay` (optional, defaults to 1) seconds in between the calls.
 # Note that execution isn't aborted exactly after `timeout`
@@ -11,9 +20,9 @@ wait_until() {
     timeout="${1}"
     check_expr="${2}"
     delay="${3:-1}"
-    timeout_at=$(expr $(date +%s) + ${timeout})
+    timeout_at=$(expr $(clock_gettime_monotonic) + ${timeout})
     until eval "${check_expr}"; do
-        if [ "$(date +%s)" -ge "${timeout_at}" ]; then
+        if [ "$(clock_gettime_monotonic)" -ge "${timeout_at}" ]; then
             return 1
         fi
         sleep ${delay}
diff --git a/config/chroot_local-includes/usr/local/sbin/restart-tor b/config/chroot_local-includes/usr/local/sbin/restart-tor
index 51b494e..7153f69 100755
--- a/config/chroot_local-includes/usr/local/sbin/restart-tor
+++ b/config/chroot_local-includes/usr/local/sbin/restart-tor
@@ -2,7 +2,7 @@
 
 set -e
 
-# Import try_for()
+# Import try_for() and clock_gettime_monotonic()
 . /usr/local/lib/tails-shell-library/common.sh
 
 # Import tor_bootstrap_progress()
@@ -29,7 +29,7 @@ service tor restart
 #     options  set by Vidalia will be lost since they weren't written to torrc.
 
 bootstrap_progress=0
-last_bootstrap_change=$(date +%s)
+last_bootstrap_change=$(clock_gettime_monotonic)
 
 maybe_restart_tor() {
     local new_bootstrap_progress=$(tor_bootstrap_progress)
@@ -38,14 +38,14 @@ maybe_restart_tor() {
         return 0
     elif [ $new_bootstrap_progress -gt $bootstrap_progress ]; then
         bootstrap_progress=$new_bootstrap_progress
-        last_bootstrap_change=$(date +%s)
+        last_bootstrap_change=$(clock_gettime_monotonic)
         return 1
-    elif [ $(expr $(date +%s) - $last_bootstrap_change) -ge 20 ]; then
+    elif [ $(expr $(clock_gettime_monotonic) - $last_bootstrap_change) -ge 20 ]; then
         log "Tor seems to have stalled while bootstrapping. Restarting Tor."
         clear_tor_log
         service tor restart
         bootstrap_progress=0
-        last_bootstrap_change=$(date +%s)
+        last_bootstrap_change=$(clock_gettime_monotonic)
         return 1
     else
         return 1
